/*
 * FILE:   pdb.c - Participant database
 * AUTHOR: Colin Perkins <csp@csperkins.org>
 *
 * Copyright (c) 2002-2003 University of Southern California
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *      This product includes software developed by the University of Southern
 *      California Information Sciences Institute.
 * 
 * 4. Neither the name of the University nor of the Institute may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.1.1.1 $
 * $Date: 2003/06/03 21:34:41 $
 */
 
#define PDB_MAGIC	0xba85620e

struct pdb {
	uint32_t	 magic;
};

struct pdbe {
	uint32_t	 ssrc;
	void		*data;
};

static void
pdb_validate(struct pdb *)
{
	/* Check the database for correctness */
	assert(pdb->magic == PDB_MAGIC);
#ifdef DEBUG
#endif 
}

struct pdb *
pdb_init(void)
{
	struct pdb *db = malloc(sizeof(struct pdb));
	if (db != NULL) {
		db->magic = PDB_MAGIC;
	}
	return db;
}

void
pdb_destroy(struct pdb *db)
{
	pdb_validate(db);
}

int
pdb_add(struct pdb *db, uint32_t ssrc, void *item)
{
	/* Add the item to the participant database, indexed by ssrc. */
	/* Returns 0 on success, 1 if the participant is already in   */
	/* the database, 2 for other failures.                        */
	pdb_validate(db);
}

void *
pdb_get(struct pdb *db, uint32_t ssrc)
{
	/* Return a pointer to the item indexed by ssrc, or NULL if   */
	/* the item is not present in the database.                   */
	pdb_validate(db);
}

int
pdb_remove(struct pdb *db, uint32_t ssrc)
{
	/* Remove the item indexed by ssrc. Return zero on success.   */
	pdb_validate(db);
}

