/*
 * FILE:     rtp_callback.c
 * AUTHOR:   Colin Perkins <csp@csperkins.org>
 * MODIFIED: N.Cihan Tas
 *           Ladan Gharai
 *
 * Copyright (c) 2001-2003 University of Southern California
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *      This product includes software developed by the University of Southern
 *      California Information Sciences Institute.
 * 
 * 4. Neither the name of the University nor of the Institute may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.32 $
 * $Date: 2004/04/30 14:28:07 $
 *
 */

#include "config.h"
#include "config_unix.h"
#include "config_win32.h"
#include "debug.h"
#include "video_types.h"
#include "video_display.h"
#include "video_convert/colour_convert.h"
#include "ntp.h"
#include "tv.h"
#include "rtp/rtp.h"
#include "rtp/rtp_callback.h"
#include "tfrc.h"
#include "rtp/pbuf.h"

extern char		*frame;

char		 	 hdr_buf[100];
struct msghdr		 msg;
struct iovec		 iov[10];

extern uint32_t  RTT;

static void
handle_rr(struct rtp *session, rtp_event *e)
{
	float		fract_lost, tmp;
        uint32_t	ntp_sec, ntp_frac, now; 
	rtcp_rr		*r = (rtcp_rr *) e->data;

	if (e->ssrc == rtp_my_ssrc(session)) {
		/* Filter out loopback reports */
		return;
	}

	fract_lost = (r->fract_lost * 100.0) / 256.0;
	printf("Got RR for 0x%08x loss fraction = %f jitter=%d\n", r->ssrc, fract_lost, r->jitter);

        /* Compute RTT=now-lsr-dlsr */
	if (r->lsr != 0) {
		ntp64_time(&ntp_sec, &ntp_frac);
		now = ntp64_to_ntp32(ntp_sec, ntp_frac); 

		debug_msg("%u %u %u = %u\n", now, r->lsr, r->dlsr, now - r->lsr - r->dlsr);

		if ((now < r->lsr) || (now - r->lsr < r->dlsr)) {
			debug_msg("Bogus RTT (packet arrived before it was sent\n");
		} else {
			tmp = ((float) (now - r->lsr - r->dlsr))/65536.0;  /* RTT in  seconds */
			RTT = tmp*1000000; /* RTT in usec */
		}
	}
	UNUSED(session);
}

void
rtp_recv_callback(struct rtp *session, rtp_event *e)
{
	rtcp_app	*pckt_app = (rtcp_app *)   e->data;
	rtp_packet	*pckt_rtp   = (rtp_packet *) e->data;
//	struct tfrc	*tfrc_state = (struct tfrc *) rtp_get_userdata(session);
	struct pbuf	*playout_buf = (struct pbuf *) rtp_get_userdata(session);
	struct timeval	 curr_time;

	switch (e->type) {
	case RX_RTP:
		pbuf_insert(playout_buf, pckt_rtp);
		//tfrc_recv_data(tfrc_state, curr_time, pckt_rtp->seq, pckt_rtp->data_len + 40);
		break;
	case RX_RTCP_START:
		break;
	case RX_RTCP_FINISH:
		break;
	case RX_SR:
		break;
	case RX_RR:
		handle_rr(session, e);
		break;
	case RX_RR_EMPTY:
		break;
	case RX_SDES:
		break;
	case RX_APP:
	        pckt_app   = (rtcp_app *)   e->data;
		if (strncmp(pckt_app->name, "RTT_", 4) == 0) {
			assert(pckt_app->length  = 3);
			assert(pckt_app->subtype = 0);
			gettimeofday(&curr_time, NULL);
//			tfrc_recv_rtt(tfrc_state, curr_time, ntohl(*((int *) pckt_app->data)));
		}
		break;
	case RX_BYE:
		break;
	case SOURCE_DELETED:
		break;
	case SOURCE_CREATED:
		break;
	case RR_TIMEOUT:
		break;
	default:
		debug_msg("Unknown RTP event (type=%d)\n", e->type);
	}
}

